package com.aidex.common.app.server.impl;

import com.aidex.common.app.domain.vo.SysWharfVO;
import com.aidex.common.app.server.IAppService;
import com.aidex.common.exception.SysException;
import com.aidex.common.gps.server.IGpsService;
import com.aidex.common.plush.domain.vo.SysShipExtendVO;
import com.aidex.common.plush.domain.vo.SysShipResVO;
import com.aidex.common.plush.service.SysShipExtendService;
import com.aidex.common.utils.StringUtils;
import com.aidex.common.utils.dist.CalcDist;
import com.aidex.framework.cache.ConfigUtils;
import com.aidex.system.domain.SysNotice;
import com.aidex.system.domain.SysWharf;
import com.aidex.system.service.SysNoticeService;
import com.aidex.system.service.SysShipService;
import com.aidex.system.service.SysWharfService;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class AppService implements IAppService {

    @Autowired
    private SysNoticeService sysNoticeService;
    @Autowired
    private SysWharfService sysWharfService;
    @Autowired
    private IGpsService iGpsService;

    @Autowired
    private SysShipService sysShipService;
    @Autowired
    private SysShipExtendService sysShipExtendService;


    public List<SysNotice> getCarouselList(){

        List<SysNotice> carouselList = sysNoticeService.findList(new SysNotice(){{setCarousel("1");}});
        return carouselList;
    }

    public List<SysNotice> getNoticeList(){
        List<SysNotice> noticeList = sysNoticeService.findList(new SysNotice(){{setCarousel("0");setNoticeType("1");}});
        return noticeList;
    }

    public SysNotice findCarouselAndNoticeDetails(String id){
        return sysNoticeService.get(id);
    }

    public List<SysWharf> findSysWharfList(){
        return sysWharfService.findList(new SysWharf());
    }

    /*
    * direction 方向 true正向 false反向
    * */
    public List<SysShipExtendVO> findAllLocation(Boolean direction){
        List<SysShipExtendVO> sysShipExtendVOS = sysShipExtendService.findAllonline(direction?"1":"0");
        if(sysShipExtendVOS.size() <= 0)
            throw new SysException(8821,"暂无运行船只");
        return sysShipExtendVOS;
    }


    public SysShipResVO findAllLocationByNext(Boolean direction,String lat, String lon){
        if(StringUtils.isEmpty(lat) || StringUtils.isEmpty(lon))
            throw new SysException(4441,"请告诉我你在哪");

        String travel = direction?"1":"0";
        // 离我最近的站点
        SysWharf wharf = this.findNearestStation(lat,lon);

        if(!wharf.getStatus().equals("0"))
            throw new SysException(4442, wharf.getWharfNanme()+" 已经停用！请更换其他站点");

        List<SysShipExtendVO> sysShipExtendVOS = sysShipExtendService.findAllonline(travel).stream().map(lot -> {
                    lot.setDistance(CalcDist.calculateDistance(Double.valueOf(wharf.getLat()), Double.valueOf(wharf.getLng()),
                            Double.valueOf(lot.getLat()), Double.valueOf(lot.getLon())));
                    return lot;
                }).sorted(Comparator.comparing(SysShipExtendVO::getDistance))
                .collect(Collectors.toList());

        if(sysShipExtendVOS.size() <= 0)
            throw new SysException(8821,"暂无运行船只");


        // 默认到站停留时间
        Integer inStopTime = Integer.valueOf(ConfigUtils.getConfigByKey("pull.in.time").getConfigValue());
        // 停靠速度
        Integer stopSpeed = Integer.valueOf(ConfigUtils.getConfigByKey("stop.speed").getConfigValue());

        // 查找未经过该站点的船只
        SysShipExtendVO noSiteShip = null;
        for (SysShipExtendVO sysShipExtendVO : sysShipExtendVOS) {
            if(direction && CalcDist.gps2d(Double.valueOf(wharf.getLat()),Double.valueOf(wharf.getLng()),Double.valueOf(sysShipExtendVO.getLat()),Double.valueOf(sysShipExtendVO.getLon())) <= 0){
                noSiteShip = sysShipExtendVO;
                break;
            }
            if(!direction && CalcDist.gps2d(Double.valueOf(sysShipExtendVO.getLat()),Double.valueOf(sysShipExtendVO.getLon()),Double.valueOf(wharf.getLat()),Double.valueOf(wharf.getLng())) <= 0){
                noSiteShip = sysShipExtendVO;
                break;
            }
        }
        if(noSiteShip == null)
            throw new SysException(8221,"当前运行船只已全部经过该站点！暂无船只！请等待发船");
        SysShipResVO sysShipResVO = new SysShipResVO();
        BeanUtils.copyProperties(noSiteShip,sysShipResVO);
        sysShipResVO.setIsStopMyLocationSite(Boolean.FALSE);
        sysShipResVO.setMy(wharf);

        Long defaultSpeed = 0L;
        if(sysShipResVO.getTravel() == 1L)
            defaultSpeed = Long.valueOf(ConfigUtils.getConfigByKey("forward.default.speed").getConfigValue());
        else
            defaultSpeed = Long.valueOf(ConfigUtils.getConfigByKey("reverse.default.speed").getConfigValue());

        // 停靠时间
        Long awaitTime = 0L;
        Double stopDist = 0.0;
        Long distSiteNum = 0L;

        SysWharf wharfShipStop = null;
        SysWharf wharfShipNext = null;

        // 船只已经停止
        if(sysShipResVO.getIsStop() == 1L){
            wharfShipStop = sysWharfService.get(sysShipResVO.getWharfStop());
            sysShipResVO.setStop(wharfShipStop);
            // 停靠同一个站点
            if(wharfShipStop.getWharfNanme().equals(wharf.getWharfNanme())){
                sysShipResVO.setIsStopMyLocationSite(Boolean.TRUE);
            }else {
                //如果处于停止状态则计算停留剩余时间
                Double loadingTime = 0.0;
                if(sysShipResVO.getUpdateTime() != null && sysShipResVO.getUpdateTime() < System.currentTimeMillis())
                    loadingTime = inStopTime.doubleValue() - CalcDist.calcStopTime(sysShipResVO.getUpdateTime().toString());
                // 计算站点间距离
                stopDist = CalcDist.calculateDistance(Double.valueOf(wharf.getLat()), Double.valueOf(wharf.getLng()),Double.valueOf(wharfShipStop.getLat()), Double.valueOf(wharfShipStop.getLng()));
                Double time = Math.ceil(((stopDist / 100) / Double.valueOf(defaultSpeed)) * 60);
                awaitTime = time.longValue() + loadingTime.longValue();
                // 船只正向反向距离站点数量
                Long myNum = wharf.getWharfOrder() / 10;
                Long shipNum = wharfShipStop.getWharfOrder() / 10;
                if(sysShipResVO.getTravel() == 1L)
                    distSiteNum = shipNum - myNum;
                else
                    distSiteNum = myNum - shipNum;
            }
        }else {
            wharfShipNext = sysWharfService.get(sysShipResVO.getNextWharf());
            stopDist = CalcDist.calculateDistance(Double.valueOf(wharf.getLat()), Double.valueOf(wharf.getLng()),Double.valueOf(sysShipResVO.getLat()), Double.valueOf(sysShipResVO.getLon()));
            Double time = 0.0;
            if(Double.valueOf(sysShipResVO.getSpeed()) > Double.valueOf(stopSpeed))
                time = Math.ceil(((stopDist / 100) / Double.valueOf(sysShipResVO.getSpeed())) * 60);
            else
                time = Math.ceil(((stopDist / 100) / Double.valueOf(defaultSpeed)) * 60);
            awaitTime = time.longValue();
            sysShipResVO.setNext(wharfShipNext);
            // 船只正向反向距离站点数量
            Long myNum = wharf.getWharfOrder() / 10;
            Long shipNum = wharfShipNext.getWharfOrder() / 10;
            if(sysShipResVO.getTravel() == 1L)
                distSiteNum = shipNum - myNum;
            else
                distSiteNum = myNum - shipNum;
        }
        // fixme 核载人数 此处需减去摄像头获得数据
        Long resetNum = Long.valueOf(sysShipResVO.getNuclearLoadNum());

        sysShipResVO.setReseat(resetNum);
        sysShipResVO.setAwaitTime(awaitTime);
        sysShipResVO.setDistance(stopDist);
        sysShipResVO.setDistSiteNum(distSiteNum);

        return sysShipResVO;
    }

    public SysWharfVO findNearestStation(String lat, String lon){
        if(StringUtils.isEmpty(lat) || StringUtils.isEmpty(lon))
            throw new SysException(4441,"请告诉我你在哪");
        List<SysWharf> listWharf = sysWharfService.findList(new SysWharf());
        List<SysWharfVO> vos = new ArrayList<>();
        for (SysWharf sysWharf : listWharf) {
            SysWharfVO wharfVO = new SysWharfVO();
            BeanUtils.copyProperties(sysWharf,wharfVO);
            wharfVO.setDistance(CalcDist.calculateDistance(Double.valueOf(lat),Double.valueOf(lon),
                    Double.valueOf(sysWharf.getLat()),Double.valueOf(sysWharf.getLng())));
            vos.add(wharfVO);
        }
        return  vos.stream().sorted(Comparator.comparing(SysWharfVO::getDistance)).findFirst().orElse(null);
    }

}
